<?php

// If this file is called directly, abort.

if ( ! defined( 'WPINC' ) ) {
    die;
}

/** Basic Override **/

function lastudio_elementor_recreate_editor_document_file(){

    global $wp_filesystem;
    if (empty($wp_filesystem)) {
        require_once(ABSPATH . '/wp-admin/includes/file.php');
        WP_Filesystem();
    }

    $wp_upload_dir = wp_upload_dir( null, false );
    $target_source_folder = $wp_upload_dir['basedir'] . '/elementor/';
    $target_source_file = $target_source_folder . '/editor-document.min.js';
    $remote_source_file = plugin_dir_path(LASTUDIO_PLUGIN_PATH) . 'elementor/assets/js/editor-document.min.js';

    if($wp_filesystem->exists($remote_source_file)){
        $file_content = $wp_filesystem->get_contents($remote_source_file);
        $string_search = array(
            'this.stylesheet.addDevice("mobile",0).addDevice("tablet",e.md).addDevice("desktop",e.lg)'
        );
        $string_replace = array(
            'this.stylesheet.addDevice("mobile",0).addDevice("tabletportrait",e.sm).addDevice("tablet",e.md).addDevice("laptop",e.lg).addDevice("desktop",e.xl)'
        );
        $new_content = str_replace($string_search, $string_replace, $file_content);

        if(!$wp_filesystem->is_dir($target_source_folder)){
            if(! wp_mkdir_p( $target_source_folder )){
                return new WP_Error( 'lastudio_elementor.cannot_put_contents', __( 'Cannot put contents', 'lastudio' ) );
            }
        }

        if($wp_filesystem->put_contents($target_source_file, $new_content)){
            update_option('lastudio-has-override-elementor-editor-js', true);
            return true;
        }
        else{
            update_option('lastudio-has-override-elementor-editor-js', false);
            return new WP_Error( 'lastudio_elementor.cannot_put_contents', __( 'Cannot put contents', 'lastudio' ) );
        }
    }
    else{
        return new WP_Error( 'lastudio_elementor.resource_not_exists', __( 'Resource does not exist', 'lastudio' ) );
    }
}

function lastudio_elementor_recreate_editor_file( ){

    global $wp_filesystem;
    if (empty($wp_filesystem)) {
        require_once(ABSPATH . '/wp-admin/includes/file.php');
        WP_Filesystem();
    }

    $wp_upload_dir = wp_upload_dir( null, false );
    $target_source_folder = $wp_upload_dir['basedir'] . '/elementor/';
    $target_source_file = $target_source_folder . '/editor.min.js';
    $remote_source_file = plugin_dir_path(LASTUDIO_PLUGIN_PATH) . 'elementor/assets/js/editor.min.js';

    if($wp_filesystem->exists($remote_source_file)){
        $file_content = $wp_filesystem->get_contents($remote_source_file);
        $string_search = array(
            '["desktop","tablet","mobile"]',
            '$e.routes.saveState("library"),(0,u.default)((0,s.default)(_default.prototype),"activateTab",this).call(this,e)',
            'defaultTabs(){return{"templates/blocks":',
        );
        $string_replace = array(
            '["desktop","laptop","tablet","tabletportrait","mobile"]',
            '$e.routes.saveState("library"),(0,u.default)((0,s.default)(_default.prototype),"activateTab",this).call(this,e),jQuery(document).trigger("LaStudio/Elementor/TemplateLibraryActiveTab", [e])',
            'defaultTabs(){return{"templates/lastudio":{title:"Demos",filter:{source:"lastudio"}},"templates/blocks":',
        );
        $new_content = str_replace($string_search, $string_replace, $file_content);

        $new_content = preg_replace(
            '/stylesheet\.addDevice\((.*)\)},addStyleRules/',
            'stylesheet.addDevice("mobile",0).addDevice("tabletportrait",elementorFrontend.config.breakpoints.sm).addDevice("tablet",elementorFrontend.config.breakpoints.md).addDevice("laptop",elementorFrontend.config.breakpoints.lg).addDevice("desktop",elementorFrontend.config.breakpoints.xl)},addStyleRules',
            $new_content
        );

        if(!$wp_filesystem->is_dir($target_source_folder)){
            if(! wp_mkdir_p( $target_source_folder )){
                return new WP_Error( 'lastudio_elementor.cannot_put_contents', __( 'Cannot put contents', 'lastudio' ) );
            }
        }

        if($wp_filesystem->put_contents($target_source_file, $new_content)){
            update_option('lastudio-has-override-elementor-editor-js', true);
            return true;
        }
        else{
            update_option('lastudio-has-override-elementor-editor-js', false);
            return new WP_Error( 'lastudio_elementor.cannot_put_contents', __( 'Cannot put contents', 'lastudio' ) );
        }
    }
    else{
        return new WP_Error( 'lastudio_elementor.resource_not_exists', __( 'Resource does not exist', 'lastudio' ) );
    }
}

add_action('lastudio_elementor_recreate_editor_file', 'lastudio_elementor_recreate_editor_file');

add_action('elementor/core/files/clear_cache', 'lastudio_elementor_recreate_editor_file');

add_filter('elementor/editor/localize_settings', function ($configs){

    $key = ['elementor_site', 'docs_elementor_site', 'help_the_content_url', 'help_right_click_url', 'help_flexbox_bc_url', 'elementPromotionURL', 'dynamicPromotionURL'];
    $key2 = ['help_preview_error_url', 'help_preview_http_error_url', 'help_preview_http_error_500_url'];
    if(is_array($configs)){
        foreach ($configs as $k => $v){
            if(in_array($k, $key)){
                $configs[$k] = $v . '?ref=14171';
            }
            if( $k == 'preview' && is_array($v) ){
                foreach ($v as $k1 => $v1){
                    if(in_array($k1, $key2)){
                        $v[$k1] = $v1 . '?ref=14171';
                    }
                }
                $configs[$k] = $v;
            }
        }
    }
    return $configs;
});

add_action('elementor/app/init', function (){
    add_action('wp_print_footer_scripts', function (){
        ?>
        <script type="text/javascript">
            (function($) {
                'use strict';
                function dpv_add_query_args( url, key, value ) {
                    var re = new RegExp("([?&])" + key + "=.*?(&|$)", "i");
                    var separator = url.indexOf('?') !== -1 ? "&" : "?";
                    if (url.match(re)){
                        return url.replace(re, '$1' + key + "=" + value + '$2');
                    }
                    else{
                        return url + separator + key + "=" + value;
                    }
                }
                function dpv_change_elementor_ref_url(){
                    $('a[href*="elementor.com"]').each(function () {
                        var _old = $(this).attr('href');
                        $(this).attr('href', dpv_add_query_args(_old, 'ref', '14171'));
                    })
                }
                $(function(){
                    dpv_change_elementor_ref_url();
                });

            })(jQuery);
        </script>
        <?php
    }, 999);
});
add_action('admin_footer', function (){
?>
    <script type="text/javascript">
        (function($) {
            'use strict';
            function dpv_add_query_args( url, key, value ) {
                var re = new RegExp("([?&])" + key + "=.*?(&|$)", "i");
                var separator = url.indexOf('?') !== -1 ? "&" : "?";
                if (url.match(re)){
                    return url.replace(re, '$1' + key + "=" + value + '$2');
                }
                else{
                    return url + separator + key + "=" + value;
                }
            }
            function dpv_change_elementor_ref_url(){
                $('a[href*="elementor.com"]').each(function () {
                    var _old = $(this).attr('href');
                    $(this).attr('href', dpv_add_query_args(_old, 'ref', '14171'));
                })
            }
            $(function(){
                dpv_change_elementor_ref_url();
            });

        })(jQuery);
    </script>
    <?php
}, 999);

add_filter('wp_redirect', function ($location, $status){
    if(strpos($location, 'elementor.com') !== false){
        $location = add_query_arg( 'ref', '14171', $location);
    }
    return $location;
}, 10, 2);

add_action( 'upgrader_process_complete', function( $upgrader_object, $options ){
    $target_plugin = 'elementor/elementor.php';
    if( $options['action'] == 'update' && $options['type'] == 'plugin' && isset( $options['plugins'] ) ) {
        foreach( $options['plugins'] as $plugin ) {
            if( $plugin == $target_plugin ) {
                do_action('lastudio_elementor_recreate_editor_file');
            }
        }
    }
}, 10, 2 );

function lastudio_elementor_override_editor_before_enqueue_scripts( $src, $handler ){
    if($handler == 'elementor-editor'){
        $wp_upload_dir = wp_upload_dir( null, false );
        return $wp_upload_dir['baseurl'] . '/elementor/editor.min.js';
    }
//    if($handler == 'elementor-editor-document'){
//        $wp_upload_dir = wp_upload_dir( null, false );
//        return $wp_upload_dir['baseurl'] . '/elementor/editor-document.min.js';
//    }
    return $src;
}

function lastudio_elementor_override_editor_wp_head(){
    ?>
    <script type="text/template" id="tmpl-elementor-control-responsive-switchers">
        <div class="elementor-control-responsive-switchers">
            <#
            var devices = responsive.devices || [ 'desktop', 'laptop', 'tablet', 'tabletportrait', 'mobile' ];
            _.each( devices, function( device ) { #>
            <a class="elementor-responsive-switcher elementor-responsive-switcher-{{ device }}" data-device="{{ device }}">
                <i class="eicon-device-{{ device }}"></i>
            </a>
            <# } );
            #>
        </div>
    </script>
    <?php
}

function lastudio_load_override_elementor_file(){
    if(defined('ELEMENTOR_VERSION')){

        $wp_upload_dir = wp_upload_dir( null, false );
        $target_source_file = $wp_upload_dir['basedir'] . '/elementor/editor.min.js';
        if(!file_exists($target_source_file)){
            do_action('lastudio_elementor_recreate_editor_file');
        }

        add_action('script_loader_src', 'lastudio_elementor_override_editor_before_enqueue_scripts', 10, 2);
        add_action('elementor/editor/wp_head', 'lastudio_elementor_override_editor_wp_head' );

        if(version_compare(ELEMENTOR_VERSION, '3.0.0', '<')){
            require LASTUDIO_PLUGIN_PATH . 'includes/extensions/elementor/override/old/includes/base/controls-stack.php';
            require LASTUDIO_PLUGIN_PATH . 'includes/extensions/elementor/override/old/core/files/css/base.php' ;
            require LASTUDIO_PLUGIN_PATH . 'includes/extensions/elementor/override/old/core/responsive/responsive.php';
        }
        else{
            require LASTUDIO_PLUGIN_PATH . 'includes/extensions/elementor/override/includes/base/controls-stack.php';
            require LASTUDIO_PLUGIN_PATH . 'includes/extensions/elementor/override/core/files/css/base.php' ;
            require LASTUDIO_PLUGIN_PATH . 'includes/extensions/elementor/override/core/responsive/responsive.php';
        }
    }
}
add_action('plugins_loaded', 'lastudio_load_override_elementor_file');

function lastudio_elementor_get_widgets_black_list( $black_list ){
    $new_black_list = array(
        'WP_Widget_Calendar',
        'WP_Widget_Pages',
        'WP_Widget_Archives',
        'WP_Widget_Media_Audio',
        'WP_Widget_Media_Image',
        'WP_Widget_Media_Gallery',
        'WP_Widget_Media_Video',
        'WP_Widget_Meta',
        'WP_Widget_Text',
        'WP_Widget_RSS',
        'WP_Widget_Custom_HTML',
        'RevSliderWidget',
        'LaStudio_Widget_Recent_Posts',
        'LaStudio_Widget_Product_Sort_By',
        'LaStudio_Widget_Price_Filter_List',
        'LaStudio_Widget_Product_Tag',
        'WP_Widget_Recent_Posts',
        'WP_Widget_Recent_Comments',
        'WC_Widget_Cart',
        'WC_Widget_Layered_Nav_Filters',
        'WC_Widget_Layered_Nav',
        'WC_Widget_Price_Filter',
        'WC_Widget_Product_Search',
        'WC_Widget_Product_Tag_Cloud',
        'WC_Widget_Products',
        'WC_Widget_Recently_Viewed',
        'WC_Widget_Top_Rated_Products',
        'WC_Widget_Recent_Reviews',
        'WC_Widget_Rating_Filter'
    );

    $new_black_list = array_merge($black_list, $new_black_list);

    return $new_black_list;
}

add_filter('elementor/widgets/black_list', 'lastudio_elementor_get_widgets_black_list', 20);

add_action( 'elementor/editor/before_enqueue_scripts', function(){
    wp_enqueue_script(
        'lastudio-elementor-backend',
        LASTUDIO_PLUGIN_URL . 'public/element/js/editor-backend.js' ,
        ['jquery'],
        LASTUDIO_VERSION,
        true
    );

    $breakpoints = [
        'laptop' => [
            'name' => __( 'Laptop', 'lastudio' ),
            'text' => __( 'Preview for 1366px', 'lastudio' )
        ],
        'tablet' => [
            'name' => __( 'Tablet Landscape', 'lastudio' ),
            'text' => __( 'Preview for 1024px', 'lastudio' )
        ],
        'tabletportrait' => [
            'name' => __( 'Tablet Portrait', 'lastudio' ),
            'text' => __( 'Preview for 768px', 'lastudio' )
        ]
    ];
    if(la_is_local()){
        $breakpoints = [
            'laptop1' => [
                'name' => __( 'Laptop 1', 'lastudio' ),
                'text' => __( 'Preview for 1680px', 'lastudio' )
            ],
            'laptop2' => [
                'name' => __( 'Laptop 2', 'lastudio' ),
                'text' => __( 'Preview for 1440px', 'lastudio' )
            ],
            'laptop' => [
                'name' => __( 'Laptop', 'lastudio' ),
                'text' => __( 'Preview for 1366px', 'lastudio' )
            ],
            'tablet' => [
                'name' => __( 'Tablet Landscape', 'lastudio' ),
                'text' => __( 'Preview for 1024px', 'lastudio' )
            ],
            'tabletportrait' => [
                'name' => __( 'Tablet Portrait', 'lastudio' ),
                'text' => __( 'Preview for 768px', 'lastudio' )
            ]
        ];
    }

    wp_localize_script('lastudio-elementor-backend', 'LaCustomBPFE', $breakpoints);
} );

/** Add Pro config to widget **/

/**
 * Enqueue Monion & Sticky Scripts
 */
add_action('elementor/frontend/before_enqueue_scripts', function(){

    if(defined('ELEMENTOR_PRO_VERSION')){
        return;
    }

    wp_register_script(
        'lastudio-sticky',
        LASTUDIO_PLUGIN_URL . 'public/element/js/lib/jquery.sticky.min.js',
        [
            'jquery',
        ],
        LASTUDIO_VERSION,
        true
    );

    wp_enqueue_script(
        'lastudio-motion-fx',
        LASTUDIO_PLUGIN_URL . 'public/element/js/lib/motion-fx.min.js' ,
        [
            'elementor-frontend-modules',
            'lastudio-sticky'
        ],
        LASTUDIO_VERSION,
        true
    );
});

add_action('elementor/element/after_section_end', function( $controls_stack, $section_id ){

    if ( 'section_custom_css_pro' !== $section_id || defined('ELEMENTOR_PRO_VERSION') ) {
        return;
    }

    $old_section = \Elementor\Plugin::instance()->controls_manager->get_control_from_stack( $controls_stack->get_unique_name(), 'section_custom_css_pro' );
    \Elementor\Plugin::instance()->controls_manager->remove_control_from_stack( $controls_stack->get_unique_name(), [ 'section_custom_css_pro', 'custom_css_pro' ] );

    $controls_stack->start_controls_section(
        'section_custom_css',
        [
            'label' => __( 'Custom CSS', 'lastudio' ),
            'tab' => $old_section['tab'],
        ]
    );

    $controls_stack->add_control(
        'custom_css',
        [
            'type' => \Elementor\Controls_Manager::CODE,
            'label' => __( 'Add your own custom CSS here', 'lastudio' ),
            'language' => 'css',
            'description' => __( 'Use "selector" to target wrapper element. Examples:<br>selector {color: red;} // For main element<br>selector .child-element {margin: 10px;} // For child element<br>.my-class {text-align: center;} // Or use any custom selector', 'lastudio' ),
            'render_type' => 'ui',
            'separator' => 'none'
        ]
    );

    $controls_stack->end_controls_section();

}, 10, 2);

add_action('elementor/element/parse_css', function( $post_css, $element ){

    if(defined('ELEMENTOR_PRO_VERSION')){
        return;
    }

    if ( $post_css instanceof \Elementor\Core\DynamicTags\Dynamic_CSS) {
        return;
    }
    $element_settings = $element->get_settings();

    if ( empty( $element_settings['custom_css'] ) ) {
        return;
    }

    $css = trim( $element_settings['custom_css'] );

    if ( empty( $css ) ) {
        return;
    }
    $css = str_replace( 'selector', $post_css->get_element_unique_selector( $element ), $css );

    // Add a css comment
    $css = sprintf( '/* Start custom CSS for %s, class: %s */', $element->get_name(), $element->get_unique_selector() ) . $css . '/* End custom CSS */';

    $post_css->get_stylesheet()->add_raw_css( $css );
}, 10, 2);

add_action('elementor/css-file/post/parse', function( $post_css ){

    if(defined('ELEMENTOR_PRO_VERSION')){
        return;
    }

    $document = \Elementor\Plugin::instance()->documents->get( $post_css->get_post_id() );
    $custom_css = $document->get_settings( 'custom_css' );

    $custom_css = trim( $custom_css );

    if ( empty( $custom_css ) ) {
        return;
    }

    $custom_css = str_replace( 'selector', $document->get_css_wrapper_selector(), $custom_css );

    // Add a css comment
    $custom_css = '/* Start custom CSS */' . $custom_css . '/* End custom CSS */';

    $post_css->get_stylesheet()->add_raw_css( $custom_css );

});

function lastudio_element_add_controls_group_to_element( $element ){
    if(defined('ELEMENTOR_PRO_VERSION')){
        return;
    }

    $exclude = [];
    $selector = '{{WRAPPER}}';
    if ( $element instanceof \Elementor\Element_Section ) {
        $exclude[] = 'motion_fx_mouse';
    }
    elseif ( $element instanceof \Elementor\Element_Column ) {
        $selector .= ' > .elementor-column-wrap';
    }
    else {
        $selector .= ' > .elementor-widget-container';
    }
    $element->add_group_control(
        'motion_fx',
        [
            'name' => 'motion_fx',
            'selector' => $selector,
            'exclude' => $exclude,
        ]
    );
}

add_action( 'elementor/element/section/section_effects/after_section_start', 'lastudio_element_add_controls_group_to_element' );
add_action( 'elementor/element/column/section_effects/after_section_start', 'lastudio_element_add_controls_group_to_element' );
add_action( 'elementor/element/common/section_effects/after_section_start', 'lastudio_element_add_controls_group_to_element' );

function lastudio_element_add_controls_group_to_element_background( $element ){
    if(defined('ELEMENTOR_PRO_VERSION')){
        return;
    }

    $element->start_injection( [
        'of' => 'background_bg_width_mobile',
    ] );

    $element->add_group_control(
        'motion_fx',
        [
            'name' => 'background_motion_fx',
            'exclude' => [
                'rotateZ_effect',
                'tilt_effect',
                'transform_origin_x',
                'transform_origin_y',
            ],
        ]
    );

    $options = [
        'separator' => 'before',
        'conditions' => [
            'relation' => 'or',
            'terms' => [
                [
                    'terms' => [
                        [
                            'name' => 'background_background',
                            'value' => 'classic',
                        ],
                        [
                            'name' => 'background_image[url]',
                            'operator' => '!==',
                            'value' => '',
                        ],
                    ],
                ],
                [
                    'terms' => [
                        [
                            'name' => 'background_background',
                            'value' => 'gradient',
                        ],
                        [
                            'name' => 'background_color',
                            'operator' => '!==',
                            'value' => '',
                        ],
                        [
                            'name' => 'background_color_b',
                            'operator' => '!==',
                            'value' => '',
                        ],
                    ],
                ],
            ],
        ],
    ];

    $element->update_control( 'background_motion_fx_motion_fx_scrolling', $options );

    $element->update_control( 'background_motion_fx_motion_fx_mouse', $options );

    $element->end_injection();
}
add_action( 'elementor/element/section/section_background/before_section_end', 'lastudio_element_add_controls_group_to_element_background' );
add_action( 'elementor/element/column/section_style/before_section_end', 'lastudio_element_add_controls_group_to_element_background' );

function lastudio_element_add_sticky_control_to_element( $element ){
    if(defined('ELEMENTOR_PRO_VERSION')){
        return;
    }
    $element->add_control(
        'sticky',
        [
            'label' => __( 'Sticky', 'lastudio' ),
            'type' => \Elementor\Controls_Manager::SELECT,
            'options' => [
                '' => __( 'None', 'lastudio' ),
                'top' => __( 'Top', 'lastudio' ),
                'bottom' => __( 'Bottom', 'lastudio' ),
            ],
            'separator' => 'before',
            'render_type' => 'none',
            'frontend_available' => true,
        ]
    );

    $element->add_control(
        'sticky_on',
        [
            'label' => __( 'Sticky On', 'lastudio' ),
            'type' => \Elementor\Controls_Manager::SELECT2,
            'multiple' => true,
            'label_block' => 'true',
            'default' => [ 'desktop', 'tablet', 'mobile' ],
            'options' => [
                'desktop' => __( 'Desktop', 'lastudio' ),
                'tablet' => __( 'Tablet', 'lastudio' ),
                'mobile' => __( 'Mobile', 'lastudio' ),
            ],
            'condition' => [
                'sticky!' => '',
            ],
            'render_type' => 'none',
            'frontend_available' => true,
        ]
    );

    $element->add_control(
        'sticky_offset',
        [
            'label' => __( 'Offset', 'lastudio' ),
            'type' => \Elementor\Controls_Manager::NUMBER,
            'default' => 0,
            'min' => 0,
            'max' => 500,
            'required' => true,
            'condition' => [
                'sticky!' => '',
            ],
            'render_type' => 'none',
            'frontend_available' => true,
        ]
    );

    $element->add_control(
        'sticky_effects_offset',
        [
            'label' => __( 'Effects Offset', 'lastudio' ),
            'type' => \Elementor\Controls_Manager::NUMBER,
            'default' => 0,
            'min' => 0,
            'max' => 1000,
            'required' => true,
            'condition' => [
                'sticky!' => '',
            ],
            'render_type' => 'none',
            'frontend_available' => true,
        ]
    );

    /*		if ( $element instanceof \Elementor\Widget_Base ) { */
    $element->add_control(
        'sticky_parent',
        [
            'label' => __( 'Stay In Column', 'lastudio' ),
            'type' => \Elementor\Controls_Manager::SWITCHER,
            'condition' => [
                'sticky!' => '',
            ],
            'render_type' => 'none',
            'frontend_available' => true,
        ]
    );
    /*		} */

    $element->add_control(
        'sticky_divider',
        [
            'type' => \Elementor\Controls_Manager::DIVIDER,
        ]
    );
}
add_action( 'elementor/element/section/section_effects/after_section_start', 'lastudio_element_add_sticky_control_to_element' );
add_action( 'elementor/element/common/section_effects/after_section_start', 'lastudio_element_add_sticky_control_to_element' );

/** Add Shortcode **/
if(!defined('ELEMENTOR_PRO_VERSION')) {
    if (is_admin()) {
        add_action('manage_' . \Elementor\TemplateLibrary\Source_Local::CPT . '_posts_columns', function ($defaults) {
            $defaults['shortcode'] = __('Shortcode', 'lastudio');
            return $defaults;
        });
        add_action('manage_' . \Elementor\TemplateLibrary\Source_Local::CPT . '_posts_custom_column', function ( $column_name, $post_id) {
            if ( 'shortcode' === $column_name ) {
                // %s = shortcode, %d = post_id
                $shortcode = esc_attr( sprintf( '[%s id="%d"]', 'elementor-template', $post_id ) );
                printf( '<input class="elementor-shortcode-input" type="text" readonly onfocus="this.select()" value="%s" />', $shortcode );
            }
        }, 10, 2);
    }
    add_shortcode( 'elementor-template', function( $attributes = [] ){
        if ( empty( $attributes['id'] ) ) {
            return '';
        }
        $include_css = false;
        if ( isset( $attributes['css'] ) && 'false' !== $attributes['css'] ) {
            $include_css = (bool) $attributes['css'];
        }
        return \Elementor\Plugin::instance()->frontend->get_builder_content_for_display( $attributes['id'], $include_css );
    } );

    add_action('elementor/init', function(){
        \Elementor\Plugin::instance()->documents->register_document_type( 'footer', \LaStudio_Element\Classes\Footer_Location::get_class_full_name() );
    });
}

add_filter('single_template', function( $template ){
    if(is_singular('elementor_library')){
        return LASTUDIO_PLUGIN_PATH .'includes/extensions/elementor/single-elementor_library.php';
    }
    return $template;
});
/** Add Footer Location **/

add_filter('lastudio-elements/custom-widget-css/parse_content', function( $css ) {
	if ( class_exists( 'Jetpack' ) && Jetpack::is_module_active( 'photon' ) ) {
		$css = preg_replace_callback(
			'~\bbackground(-image)?\s*:(.*?)\(\s*(\'|")?(?<image>.*?)\3?\s*\)~i',
			function($matches){
				return str_replace(
					$matches['image'],
					jetpack_photon_url($matches['image']),
					$matches[0]
				);
			},
			$css
		);
	}
	return $css;
});

/**   OVERRIDE Widget Base  **/
add_action('elementor/element/section/section_layout/before_section_end', 'lastudio_element_modify_content_width_for_section', 10, 2);
function lastudio_element_modify_content_width_for_section( $element, $args ){
    $element->remove_control('content_width');
    $element->add_responsive_control(
        'content_width',
        [
            'label' => __( 'Content Width', 'elementor' ),
            'type' => \Elementor\Controls_Manager::SLIDER,
            'range' => [
                'px' => [
                    'min' => 500,
                    'max' => 1600,
                ],
                '%' => [
                    'min' => 10,
                    'max' => 100,
                ],
                'vw' => [
                    'min' => 10,
                    'max' => 100,
                ],
            ],
            'size_units' => [ 'px' , '%' , 'vw' ],
            'selectors' => [
                '{{WRAPPER}} > .elementor-container' => 'max-width: {{SIZE}}{{UNIT}};',
            ],
            'condition' => [
                'layout' => [ 'boxed' ],
            ],
            'separator' => 'none',
            'show_label' => false,
        ],
        [
                'index' => 3
        ]
    );
    $element->update_control(
        'html_tag',
        [
            'default' => 'div'
        ]
    );
}

add_action('elementor/element/after_section_end', 'lastudio_element_modify_padding_margin_from_widget_common', 10, 2);
function lastudio_element_modify_padding_margin_from_widget_common( $controls_stack, $section_id ){
    if($section_id === 'section_advanced'){
        $controls_stack->update_responsive_control(
            'margin',
            [
                'size_units' => [ 'px', 'em', '%', 'rem', 'vw', 'vh' ]
            ]
        );
        $controls_stack->update_responsive_control(
            'padding',
            [
                'size_units' => [ 'px', 'em', '%', 'rem', 'vw', 'vh' ]
            ]
        );
    }

    if($section_id === '_section_style'){
        $controls_stack->update_responsive_control(
            '_margin',
            [
                'size_units' => [ 'px', 'em', '%', 'rem', 'vw', 'vh' ]
            ]
        );
        $controls_stack->update_responsive_control(
            '_padding',
            [
                'size_units' => [ 'px', 'em', '%', 'rem', 'vw', 'vh' ]
            ]
        );
    }
}

/**
 * Modify Icon List - Text Indent control
 */
add_action('elementor/element/icon-list/section_text_style/before_section_end', function( $element, $args ){
    $element->remove_control('text_indent');
    $element->update_control('icon_color', [
        'selectors' => [
            '{{WRAPPER}} .elementor-icon-list-icon i' => 'color: {{VALUE}};',
            '{{WRAPPER}} .elementor-icon-list-icon svg' => 'fill: {{VALUE}};color: {{VALUE}};',
        ]
    ]);
    $element->add_responsive_control(
        'text_indent',
        [
            'label' => __( 'Text Indent', 'elementor' ),
            'type' => \Elementor\Controls_Manager::SLIDER,
            'range' => [
                'px' => [
                    'max' => 50,
                ],
            ],
            'selectors' => [
                '{{WRAPPER}} .elementor-icon-list-text' => is_rtl() ? 'padding-right: {{SIZE}}{{UNIT}};' : 'padding-left: {{SIZE}}{{UNIT}};',
            ],
        ]
    );
}, 10, 2);

/**
 * Modify Icon - Padding & shadow
 */
add_action('elementor/element/icon/section_style_icon/before_section_end', function( $element, $args ){
    $element->remove_control('icon_padding');
    $element->add_responsive_control(
        'icon_padding',
        [
            'label' => __( 'Padding', 'elementor' ),
            'type' => \Elementor\Controls_Manager::SLIDER,
            'selectors' => [
                '{{WRAPPER}} .elementor-icon' => 'padding: {{SIZE}}{{UNIT}};',
            ],
            'range' => [
                'em' => [
                    'min' => 0,
                    'max' => 10,
                ],
                'px' => [
                    'min' => 0,
                    'max' => 200,
                ],
            ],
            'size_units' => [ 'em', 'px' ],
            'condition' => [
                'view!' => 'default',
            ],
        ]
    );
    $element->add_group_control(
        Elementor\Group_Control_Box_Shadow::get_type(),
        [
            'name'     => 'i_shadow',
            'selector' => '{{WRAPPER}} .elementor-icon',
            'condition' => [
                'view!' => 'default',
            ]
        ]
    );
}, 10, 2);

/**
 * Modify Divider - Weight control
 */
add_action('elementor/element/divider/section_divider_style/before_section_end', function( $element, $args ){
    $element->remove_control('weight');
    $element->add_responsive_control(
        'weight',
        [
            'label' => __( 'Weight', 'lastudio' ),
            'type' => \Elementor\Controls_Manager::SLIDER,
            'default' => [
                'size' => 1
            ],
            'range' => [
                'px' => [
                    'min' => 1,
                    'max' => 10,
                    'step' => 0.1,
                ],
            ],
            'render_type' => 'template',
            'selectors' => [
                '{{WRAPPER}}' => '--divider-border-width: {{SIZE}}{{UNIT}}'
            ]
        ]
    );
}, 10, 2);
/**
 * Modify Image - Max Width control
 */
add_action('elementor/element/image/section_style_image/before_section_end', function( $element, $args ){
    $element->update_responsive_control(
        'space',
        [
            'label' => __( 'Max Width', 'elementor' ),
            'size_units' => [ '%', 'px' ],
        ]
    );
}, 10, 2);

/**
 * Modify Image Box - Position control
 */
add_action('elementor/element/image-box/section_image/before_section_end', function( $element, $args ){
    $element->add_control(
        'keep_position',
        [
            'label' => __( 'Keep position on mobile', 'elementor' ),
            'type' => \Elementor\Controls_Manager::SWITCHER,
            'return_value' => 'yes',
            'default' => '',
            'condition'    => [
                'position!' => 'top'
            ],
            'prefix_class' => 'keep-mb-pos-',
        ]
    );
    $element->add_control(
        'enable_divider',
        [
            'label' => __( 'Enable Divider', 'elementor' ),
            'type' => \Elementor\Controls_Manager::SELECT,
            'options' => [
                '' => __( 'Disable', 'lastudio' ),
                'before' => __( 'Before Title', 'lastudio' ),
                'after' => __( 'After Title', 'lastudio' ),
            ],
            'prefix_class' => 'divider-ps-',
        ]
    );
    $element->add_responsive_control(
        'divider_alignment',
        array(
            'label'   => esc_html__( 'Alignment', 'lastudio' ),
            'type'    => \Elementor\Controls_Manager::CHOOSE,
            'default' => 'left',
            'options' => array(
                'left'    => array(
                    'title' => esc_html__( 'Left', 'lastudio' ),
                    'icon'  => 'eicon-h-align-left',
                ),
                'center' => array(
                    'title' => esc_html__( 'Center', 'lastudio' ),
                    'icon'  => 'eicon-h-align-center',
                ),
                'right' => array(
                    'title' => esc_html__( 'Right', 'lastudio' ),
                    'icon'  => 'eicon-h-align-right',
                ),
            ),
            'selectors' => [
                '{{WRAPPER}} .elementor-image-box-title:before' => 'margin: 0 auto; margin-{{VALUE}}: 0',
                '{{WRAPPER}} .elementor-image-box-title:after' => 'margin: 0 auto; margin-{{VALUE}}: 0',
            ],
            'condition' => [
                'enable_divider!' => '',
            ]
        )
    );
    $element->add_control(
        'divider_color',
        [
            'label' => __( 'Divider Color', 'lastudio' ),
            'type' => \Elementor\Controls_Manager::COLOR,
            'selectors' => [
                '{{WRAPPER}} .elementor-image-box-title' => '--divider-color: {{VALUE}};',
            ],
            'condition' => [
                'enable_divider!' => '',
            ]
        ]
    );
    $element->add_responsive_control(
        'divider_width',
        [
            'label' => __( 'Divider Width', 'elementor' ),
            'type' => \Elementor\Controls_Manager::SLIDER,
            'selectors' => [
                '{{WRAPPER}} .elementor-image-box-title' => '--divider-width: {{SIZE}}{{UNIT}};',
            ],
            'range' => [
                'em' => [
                    'min' => 0,
                    'max' => 10,
                ],
                'px' => [
                    'min' => 0,
                    'max' => 500,
                ],
            ],
            'size_units' => [ 'em', 'px' ],
            'condition' => [
                'enable_divider!' => '',
            ],
        ]
    );
    $element->add_responsive_control(
        'divider_height',
        [
            'label' => __( 'Divider Height', 'elementor' ),
            'type' => \Elementor\Controls_Manager::SLIDER,
            'selectors' => [
                '{{WRAPPER}} .elementor-image-box-title' => '--divider-height: {{SIZE}}{{UNIT}};',
            ],
            'range' => [
                'em' => [
                    'min' => 0,
                    'max' => 10,
                ],
                'px' => [
                    'min' => 0,
                    'max' => 20,
                ],
            ],
            'size_units' => [ 'em', 'px' ],
            'condition' => [
                'enable_divider!' => '',
            ],
        ]
    );
    $element->add_responsive_control(
        'divider_spacing',
        [
            'label'      => esc_html__( 'Divider Space', 'lastudio' ),
            'type'       => \Elementor\Controls_Manager::SLIDER,
            'size_units' => ['px', '%'],
            'selectors'  => [
                '{{WRAPPER}} .elementor-image-box-title' => '--divider-space: {{SIZE}}{{UNIT}};',
            ],
            'condition' => [
                'enable_divider!' => ''
            ]
        ]
    );
}, 10, 2);

/**
 * Modify Image Box - Width & Space controls
 */
add_action('elementor/element/image-box/section_style_image/before_section_end', function( $element, $args ){

    $element->update_responsive_control(
        'image_size',
        [
            'size_units' => [ '%', 'px' ],
        ]
    );
    $element->update_responsive_control(
        'image_space',
        [
            'selectors' => [
                '{{WRAPPER}}.elementor-position-right .elementor-image-box-img' => 'margin-left: {{SIZE}}{{UNIT}};',
                '{{WRAPPER}}.elementor-position-left .elementor-image-box-img' => 'margin-right: {{SIZE}}{{UNIT}};',
                '{{WRAPPER}}.elementor-position-top .elementor-image-box-img' => 'margin-bottom: {{SIZE}}{{UNIT}};',
                '(mobile){{WRAPPER}}:not(.keep-mb-pos-yes) .elementor-image-box-img' => 'margin-bottom: {{SIZE}}{{UNIT}};',
                '(mobile){{WRAPPER}}.elementor-position-right.keep-mb-pos-yes .elementor-image-box-img' => 'margin-left: {{SIZE}}{{UNIT}} !important;',
                '(mobile){{WRAPPER}}.elementor-position-left.keep-mb-pos-yes .elementor-image-box-img' => 'margin-right: {{SIZE}}{{UNIT}} !important;',
            ],
        ]
    );
}, 10, 2);

/**
 * Modify Icon Box - Position control
 */
add_action('elementor/element/icon-box/section_icon/before_section_end', function( $element, $args ){
    $element->add_control(
        'keep_position',
        [
            'label' => __( 'Keep position on mobile', 'elementor' ),
            'type' => \Elementor\Controls_Manager::SWITCHER,
            'return_value' => 'yes',
            'default' => '',
            'condition'    => [
                'position!' => 'top'
            ],
            'prefix_class' => 'keep-mb-pos-',
        ]
    );
    $element->add_control(
        'enable_divider',
        [
            'label' => __( 'Enable Divider', 'elementor' ),
            'type' => \Elementor\Controls_Manager::SELECT,
            'options' => [
                '' => __( 'Disable', 'lastudio' ),
                'before' => __( 'Before Title', 'lastudio' ),
                'after' => __( 'After Title', 'lastudio' ),
            ],
            'prefix_class' => 'divider-ps-',
        ]
    );
    $element->add_responsive_control(
        'divider_alignment',
        array(
            'label'   => esc_html__( 'Alignment', 'lastudio' ),
            'type'    => \Elementor\Controls_Manager::CHOOSE,
            'default' => 'left',
            'options' => array(
                'left'    => array(
                    'title' => esc_html__( 'Left', 'lastudio' ),
                    'icon'  => 'eicon-h-align-left',
                ),
                'center' => array(
                    'title' => esc_html__( 'Center', 'lastudio' ),
                    'icon'  => 'eicon-h-align-center',
                ),
                'right' => array(
                    'title' => esc_html__( 'Right', 'lastudio' ),
                    'icon'  => 'eicon-h-align-right',
                ),
            ),
            'selectors' => [
                '{{WRAPPER}} .elementor-icon-box-title:before' => 'margin: 0 auto; margin-{{VALUE}}: 0',
                '{{WRAPPER}} .elementor-icon-box-title:after' => 'margin: 0 auto; margin-{{VALUE}}: 0',
            ],
            'condition' => [
                'enable_divider!' => '',
            ]
        )
    );
    $element->add_control(
        'divider_color',
        [
            'label' => __( 'Divider Color', 'lastudio' ),
            'type' => \Elementor\Controls_Manager::COLOR,
            'selectors' => [
                '{{WRAPPER}} .elementor-icon-box-title' => '--divider-color: {{VALUE}};',
            ],
            'condition' => [
                'enable_divider!' => '',
            ]
        ]
    );
    $element->add_responsive_control(
        'divider_width',
        [
            'label' => __( 'Divider Width', 'elementor' ),
            'type' => \Elementor\Controls_Manager::SLIDER,
            'selectors' => [
                '{{WRAPPER}} .elementor-icon-box-title' => '--divider-width: {{SIZE}}{{UNIT}};',
            ],
            'range' => [
                'em' => [
                    'min' => 0,
                    'max' => 10,
                ],
                'px' => [
                    'min' => 0,
                    'max' => 500,
                ],
            ],
            'size_units' => [ 'em', 'px' ],
            'condition' => [
                'enable_divider!' => '',
            ],
        ]
    );
    $element->add_responsive_control(
        'divider_height',
        [
            'label' => __( 'Divider Height', 'elementor' ),
            'type' => \Elementor\Controls_Manager::SLIDER,
            'selectors' => [
                '{{WRAPPER}} .elementor-icon-box-title' => '--divider-height: {{SIZE}}{{UNIT}};',
            ],
            'range' => [
                'em' => [
                    'min' => 0,
                    'max' => 10,
                ],
                'px' => [
                    'min' => 0,
                    'max' => 20,
                ],
            ],
            'size_units' => [ 'em', 'px' ],
            'condition' => [
                'enable_divider!' => '',
            ],
        ]
    );
    $element->add_responsive_control(
        'divider_space',
        [
            'label'      => esc_html__( 'Divider Space', 'lastudio' ),
            'type'       => \Elementor\Controls_Manager::SLIDER,
            'size_units' => ['px', '%'],
            'selectors'  => [
                '{{WRAPPER}} .elementor-icon-box-title' => '--divider-space: {{SIZE}}{{UNIT}};',
            ],
            'condition' => [
                'enable_divider!' => ''
            ]
        ]
    );
}, 10, 2);

/**
 * Modify Icon Box - Icon Space & Padding & Shadow control
 */
add_action('elementor/element/icon-box/section_style_icon/before_section_end', function( $element, $args ){
    $element->remove_control('icon_padding');
    $element->add_responsive_control(
        'icon_padding',
        [
            'label' => __( 'Padding', 'elementor' ),
            'type' => \Elementor\Controls_Manager::SLIDER,
            'selectors' => [
                '{{WRAPPER}} .elementor-icon' => 'padding: {{SIZE}}{{UNIT}};',
            ],
            'range' => [
                'em' => [
                    'min' => 0,
                    'max' => 10,
                ],
                'px' => [
                    'min' => 0,
                    'max' => 200,
                ],
            ],
            'size_units' => [ 'em', 'px' ],
            'condition' => [
                'view!' => 'default',
            ],
        ]
    );
    $element->add_group_control(
        Elementor\Group_Control_Box_Shadow::get_type(),
        [
            'name'     => 'i_shadow',
            'selector' => '{{WRAPPER}} .elementor-icon',
            'condition' => [
                'view!' => 'default',
            ]
        ]
    );
    $element->update_responsive_control(
        'icon_space',
        [
            'selectors' => [
                '{{WRAPPER}}.elementor-position-right .elementor-icon-box-icon' => 'margin-left: {{SIZE}}{{UNIT}};',
                '{{WRAPPER}}.elementor-position-left .elementor-icon-box-icon' => 'margin-right: {{SIZE}}{{UNIT}};',
                '{{WRAPPER}}.elementor-position-top .elementor-icon-box-icon' => 'margin-bottom: {{SIZE}}{{UNIT}};',
                '(mobile){{WRAPPER}}:not(.keep-mb-pos-yes) .elementor-icon-box-icon' => 'margin-bottom: {{SIZE}}{{UNIT}};',
                '(mobile){{WRAPPER}}.elementor-position-right.keep-mb-pos-yes .elementor-icon-box-icon' => 'margin-left: {{SIZE}}{{UNIT}}!important;',
                '(mobile){{WRAPPER}}.elementor-position-left.keep-mb-pos-yes .elementor-icon-box-icon' => 'margin-right: {{SIZE}}{{UNIT}}!important;',
            ],
        ]
    );
}, 10, 2);

/**
 * Modify Heading - Color Hover
 */
add_action('elementor/element/heading/section_title_style/before_section_end', function( $element, $args ){
    $element->add_control(
        'title_hover_color',
        [
            'label' => __( 'Text Hover Color', 'elementor' ),
            'type' => \Elementor\Controls_Manager::COLOR,
            'selectors' => [
                '{{WRAPPER}} .elementor-heading-title:hover' => 'color: {{VALUE}};',
            ],
        ]
    );
}, 10, 2);

/**
 * Modify Button - Icons
 */
add_action('elementor/element/button/section_style/before_section_end', function( $element, $args ){
    $element->add_responsive_control(
        'icon_size',
        [
            'label' => __( 'Icon Size', 'lastudio' ),
            'type' => \Elementor\Controls_Manager::SLIDER,
            'range' => [
                'px' => [
                    'max' => 100
                ],
            ],
            'condition' => [
                'selected_icon[value]!' => '',
            ],
            'selectors' => [
                '{{WRAPPER}} .elementor-button .elementor-button-icon' => 'font-size: {{SIZE}}{{UNIT}}'
            ],
            'separator' => 'before',
        ]
    );
    $element->add_control(
        'icon_color',
        [
            'label' => __( 'Icon Color', 'elementor' ),
            'type' => \Elementor\Controls_Manager::COLOR,
            'selectors' => [
                '{{WRAPPER}} .elementor-button .elementor-button-icon' => 'color: {{VALUE}};'
            ],
            'condition' => [
                'selected_icon[value]!' => '',
            ],
        ]
    );
    $element->add_control(
        'icon_hover_color',
        [
            'label' => __( 'Icon Hover Color', 'elementor' ),
            'type' => \Elementor\Controls_Manager::COLOR,
            'selectors' => [
                '{{WRAPPER}} .elementor-button:hover .elementor-button-icon' => 'color: {{VALUE}};'
            ],
            'condition' => [
                'selected_icon[value]!' => '',
            ],
        ]
    );
    $element->add_responsive_control(
        'icon_margin',
        [
            'label' => __( 'Icon Margin', 'elementor' ),
            'type' => \Elementor\Controls_Manager::DIMENSIONS,
            'size_units' => [ 'px', 'em', '%' ],
            'selectors' => [
                '{{WRAPPER}} .elementor-button .elementor-button-icon' => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
            ],
            'condition' => [
                'selected_icon[value]!' => '',
            ],
        ]
    );
}, 10, 2);

/**
 * Modify Spacer - space
 */
add_action('elementor/element/spacer/section_spacer/before_section_end', function( $element, $args ){
    $element->update_responsive_control(
        'space',
        [
            'size_units' => [ 'px', 'vh', 'em', 'vw', '%' ],
        ]
    );
}, 10, 2);